@@ -249,7 +249,13 @@ setMethod("as.factor", "GenomicRanges",
         factor(as.character(x), levels=as.character(sort(unique(x))))
 )
 
-### TODO: Turn this into an S3/S4 combo for as.data.frame.GenomicRanges
+setAs("GenomicRanges", "Grouping", function(from) {
+    to <- as(grouping(seqnames(from), strand(from), start(from), end(from)),
+             "ManyToOneGrouping")
+    mcols(to)$granges <- granges(from)[end(PartitioningByEnd(to))]
+    to
+})
+
 setMethod("as.data.frame", "GenomicRanges",
     function(x, row.names=NULL, optional=FALSE, ...)
     {
@@ -310,23 +316,6 @@ setAs("GenomicRanges", "RangedData",
     }
 )
 
-.fromSeqinfoToGRanges <- function(from)
-{
-    if (any(is.na(seqlengths(from))))
-        stop("cannot create a GenomicRanges from a Seqinfo ",
-             "with NA seqlengths")
-    GRanges(seqnames(from),
-            IRanges(rep(1L, length(from)),
-                    width=seqlengths(from),
-                    names=seqnames(from)),
-            seqinfo=from)
-}
-
-setAs("Seqinfo", "GenomicRanges", .fromSeqinfoToGRanges)
-setAs("Seqinfo", "RangesList",
-    function(from) as(as(from, "GenomicRanges"), "RangesList")
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Setters.
@@ -672,9 +661,26 @@ setMethod("c", "GenomicRanges",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### "show" method.
+### Displaying
 ###
 
+.GenomicRanges_summary <- function(object)
+{
+    object_class <- class(object)
+    object_len <- length(object)
+    object_mcols <- mcols(object)
+    object_nmc <- if (is.null(object_mcols)) 0L else ncol(object_mcols)
+    paste0(object_class, " object with ", object_len, " ",
+           ifelse(object_len == 1L, "range", "ranges"),
+           " and ", object_nmc, " metadata ",
+           ifelse(object_nmc == 1L, "column", "columns"))
+}
+
+### S3/S4 combo for summary.GenomicRanges
+summary.GenomicRanges <- function(object, ...)
+    .GenomicRanges_summary(object, ...)
+setMethod("summary", "GenomicRanges", summary.GenomicRanges)
+
 .make_naked_matrix_from_GenomicRanges <- function(x)
 {
     x_len <- length(x)
@@ -706,15 +712,7 @@ show_GenomicRanges <- function(x, margin="",
                                print.classinfo=FALSE, print.seqinfo=FALSE,
                                coerce.internally.to.GRanges=TRUE)
 {
-    x_class <- class(x)
-    x_len <- length(x)
-    x_mcols <- mcols(x)
-    x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
-    cat(x_class, " object with ",
-        x_len, " ", ifelse(x_len == 1L, "range", "ranges"),
-        " and ",
-        x_nmc, " metadata ", ifelse(x_nmc == 1L, "column", "columns"),
-        ":\n", sep="")
+    cat(.GenomicRanges_summary(x), ":\n", sep="")
     ## S4Vectors:::makePrettyMatrixForCompactPrinting() assumes that head()
     ## and tail() work on 'xx'.
     if (coerce.internally.to.GRanges) {
@@ -731,7 +729,7 @@ show_GenomicRanges <- function(x, margin="",
             strand="Rle"
         )
         extraColumnNames <- extraColumnSlotNames(x)
-        .COL2CLASS <- c(.COL2CLASS, getSlots(x_class)[extraColumnNames])
+        .COL2CLASS <- c(.COL2CLASS, getSlots(class(x))[extraColumnNames])
         classinfo <-
             S4Vectors:::makeClassinfoRowForCompactPrinting(x, .COL2CLASS)
         ## A sanity check, but this should never happen!
