@@ -14,11 +14,14 @@
 ###
 
 setAs("RleList", "GRanges", function(from) {
-  rd <- as(from, "RangedData")
-  rd$strand <- "*"
-  gr <- as(rd, "GRanges")
-  seqlengths(gr) <- elementNROWS(from)
-  gr
+    score <- unlist(runValue(from), use.names=FALSE)
+    lens <- runLength(from)
+    ir <- unlist(RangesList(lapply(lens, successiveIRanges)),
+                 use.names=FALSE)
+    nrun <- lengths(lens)
+    gr <- GRanges(rep(names(from), nrun), ir, score=score)
+    seqlengths(gr) <- lengths(from)
+    gr
 })
 
 setAs("RleViewsList", "GRanges", function(from) {
@@ -67,8 +70,8 @@ bindAsGRanges <- function(...)
 
     ## First column is "runLength". Get rid of it.
     ans_mcols <- unlisted_DFL[-1L]
-    ans <- newGRanges("GRanges", ans_seqnames, ans_ranges, mcols=ans_mcols,
-                      seqlengths=ans_seqlengths)
+    ans <- new_GRanges("GRanges", ans_seqnames, ans_ranges,
+                                  mcols=ans_mcols, seqlengths=ans_seqlengths)
 
     ## Keep only ranges for which at least one variable is not NA.
     keep_idx <- which(rowSums(!is.na(mcols(ans))) != 0L)
