@@ -22,7 +22,7 @@
 
 ### 'rgl2' and 'rgl' must be List objects (typically RangesList or GRangesList)
 ### of the same length, both with a "revmap" inner metadata column.
-.fix_revmap_inner_mcol <- function(rgl2, rgl)
+.fix_inner_revmap_mcol <- function(rgl2, rgl)
 {
     unlisted_rgl2 <- unlist(rgl2, use.names=FALSE)
     unlisted_revmap2 <- mcols(unlisted_rgl2)$revmap
@@ -103,8 +103,8 @@ reconstructGRfromRGL <- function(rgl, x)
     ## Prepare 'ans_seqinfo'.
     ans_seqinfo <- seqinfo(x)
 
-    ## To be as fast as possible, we don't use internal constructor
-    ## newGRanges() and we don't check the new object.
+    ## To be as fast as possible, we don't use internal low-level constructor
+    ## new_GRanges() and we don't check the new object.
     new2("GRanges", seqnames=ans_seqnames,
                     ranges=ans_ranges,
                     strand=ans_strand,
@@ -174,28 +174,42 @@ setMethod("reduce", "GenomicRanges",
                                  min.gapwidth=min.gapwidth,
                                  with.revmap=with.revmap)
         if (with.revmap)
-            rgl2 <- .fix_revmap_inner_mcol(rgl2, rgl)
+            rgl2 <- .fix_inner_revmap_mcol(rgl2, rgl)
         reconstructGRfromRGL(rgl2, x)
     }
 )
 
-### TODO: Support the 'with.revmap' argument.
 setMethod("reduce", "GRangesList",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-             with.inframe.attrib=FALSE, ignore.strand=FALSE)
+                with.revmap=FALSE,
+                with.inframe.attrib=FALSE, ignore.strand=FALSE)
     {
         if (!identical(with.inframe.attrib, FALSE)) 
             stop("'with.inframe.attrib' argument is not supported ", 
                  "when reducing a GRangesList object")
         gr <- deconstructGRLintoGR(x)
-        ## "reduce" method for GRanges objects is fast.
         gr2 <- callGeneric(gr, drop.empty.ranges=drop.empty.ranges,
                                min.gapwidth=min.gapwidth,
+                               with.revmap=with.revmap,
                                ignore.strand=ignore.strand)
-        reconstructGRLfromGR(gr2, x)
+        ans <- reconstructGRLfromGR(gr2, x)
+
+        ## Localize 'revmap'.
+        if (with.revmap) {
+            unlisted_ans <- unlist(ans, use.names=FALSE)
+            mcols(unlisted_ans)$revmap <-
+              IRanges:::global2local_revmap(mcols(unlisted_ans)$revmap, ans, x)
+            ans <- relist(unlisted_ans, ans)
+        }
+
+        ans
     }
 )
 
+setMethod("reduce", "GenomicRangesList",
+          function(x, drop.empty.ranges = FALSE, ...) {
+              endoapply(x, reduce, drop.empty.ranges=drop.empty.ranges, ...)
+          })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### gaps()
@@ -229,20 +243,34 @@ setMethod("gaps", "GenomicRanges",
 ### Always return a GRanges *instance* whatever GenomicRanges derivative the
 ### input is, so does NOT act like an endomorphism in general. 
 setMethod("disjoin", "GenomicRanges",
-    function(x, ignore.strand=FALSE)
+    function(x, with.revmap=FALSE, ignore.strand=FALSE)
     {
-        rgl <- deconstructGRintoRGL(x, ignore.strand=ignore.strand, drop=TRUE)
-        rgl2 <- callGeneric(rgl)
+        rgl <- deconstructGRintoRGL(x, with.revmap=with.revmap, 
+                                       ignore.strand=ignore.strand, drop=TRUE)
+        rgl2 <- callGeneric(rgl, with.revmap=with.revmap)
+        if (with.revmap)
+            rgl2 <- .fix_inner_revmap_mcol(rgl2, rgl)
         reconstructGRfromRGL(rgl2, x)
     }
 )
 
 setMethod("disjoin", "GRangesList",
-    function(x, ignore.strand=FALSE)
+    function(x, with.revmap=FALSE, ignore.strand=FALSE)
     {
         gr <- deconstructGRLintoGR(x)
-        gr2 <- callGeneric(gr, ignore.strand=ignore.strand)
-        reconstructGRLfromGR(gr2, x)
+        gr2 <- callGeneric(gr, with.revmap=with.revmap,
+                               ignore.strand=ignore.strand)
+        ans <- reconstructGRLfromGR(gr2, x)
+
+        ## Localize 'revmap'.
+        if (with.revmap) {
+            unlisted_ans <- unlist(ans, use.names=FALSE)
+            mcols(unlisted_ans)$revmap <-
+              IRanges:::global2local_revmap(mcols(unlisted_ans)$revmap, ans, x)
+            ans <- relist(unlisted_ans, ans)
+        }
+
+        ans
     }
 )
 
@@ -258,7 +286,6 @@ setMethod("isDisjoint", "GenomicRanges",
         all(callGeneric(rgl))
     }
 )
-
 ### Overwrite above method with optimized method for GPos objects.
 setMethod("isDisjoint", "GPos",
     function(x, ignore.strand=FALSE) callGeneric(x@pos_runs, ignore.strand)
