@@ -6,8 +6,25 @@
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### 2 low-level helper functions.
+### 3 low-level helper functions.
 ###
+
+### A fast implementation of 'mendoapply(FUN, x, y)' for GRangesList objects.
+### Assume 'x' and 'y' are 2 GRangesList objects (not checked) of the same
+### length (checked).
+.fast_binary_mendoapply <- function(FUN, x, y, ...)
+{
+    FUN <- match.fun(FUN)
+    if (length(x) != length(y))
+        stop("'x' and 'y' must have the same length")
+    seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
+    seqlevels(y) <- seqlevels(x)
+    xgr <- deconstructGRLintoGR(x, expand.levels=TRUE)
+    ygr <- deconstructGRLintoGR(y, expand.levels=TRUE)
+    gr <- FUN(xgr, ygr, ...)
+    reconstructGRLfromGR(gr, x)
+}
+
 ### Both return a named integer vector where the names are guaranteed to be
 ### 'seqlevels(x)'.
 ###
@@ -55,41 +72,45 @@
 ### union(), intersect(), setdiff()
 ###
 
-setMethod("union", c("GRanges", "GRanges"),
+### Always return a GRanges *instance* whatever GenomicRanges derivatives are
+### passed to it (e.g. GNCList or GPos), so does NOT act like an endomorphism
+### in general.
+setMethod("union", c("GenomicRanges", "GenomicRanges"),
     function(x, y, ignore.strand=FALSE)
     {
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
+        x <- granges(x)
+        y <- granges(y)
         if (ignore.strand)
             strand(x) <- strand(y) <- "*"
-        mcols(x) <- mcols(y) <- NULL  # so we can do 'c(x, y)' below
         reduce(c(x, y), drop.empty.ranges=TRUE)
     }
 )
 
-### Equivalent to 'mendoapply(union, x, y)'.
+.unsupported_union <- function(x, y)
+    stop("union() between a ", class(x), " and a ", class(y), " object ",
+         "is not supported")
+
+setMethod("union", c("GenomicRanges", "Vector"), .unsupported_union)
+setMethod("union", c("Vector", "GenomicRanges"), .unsupported_union)
+
 setMethod("union", c("GRangesList", "GRangesList"),
-    function(x, y, ...)
-    {
-        if (length(x) != length(y))
-            stop("'x' and 'y' must have the same length")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-        seqlevels(y) <- seqlevels(x)
-        xgr <- deconstructGRLintoGR(x)
-        ygr <- deconstructGRLintoGR(y)
-        gr <- callGeneric(xgr, ygr, ...)
-        reconstructGRLfromGR(gr, x)
-    }
+    function(x, y, ...) .fast_binary_mendoapply("union", x, y, ...)
 )
 
-setMethod("intersect", c("GRanges", "GRanges"),
+### Always return a GRanges *instance* whatever GenomicRanges derivatives are
+### passed to it (e.g. GNCList or GPos), so does NOT act like an endomorphism
+### in general.
+setMethod("intersect", c("GenomicRanges", "GenomicRanges"),
     function(x, y, ignore.strand=FALSE)
     {
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
+        x <- granges(x)
+        y <- granges(y)
         if (ignore.strand)
             strand(x) <- strand(y) <- "*"
-        mcols(x) <- mcols(y) <- NULL
         seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
         ## If merge() is going to issue a warning, we don't want to get
         ## it twice.
@@ -103,29 +124,29 @@ setMethod("intersect", c("GRanges", "GRanges"),
     }
 )
 
-### Equivalent to 'mendoapply(intersect, x, y)'.
+.unsupported_intersect <- function(x, y)
+    stop("intersect() between a ", class(x), " and a ", class(y), " object ",
+         "is not supported")
+
+setMethod("intersect", c("GenomicRanges", "Vector"), .unsupported_intersect)
+setMethod("intersect", c("Vector", "GenomicRanges"), .unsupported_intersect)
+
 setMethod("intersect", c("GRangesList", "GRangesList"),
-    function(x, y, ...)
-    {
-        if (length(x) != length(y))
-            stop("'x' and 'y' must have the same length")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-        seqlevels(y) <- seqlevels(x)
-        xgr <- deconstructGRLintoGR(x)
-        ygr <- deconstructGRLintoGR(y)
-        gr <- callGeneric(xgr, ygr, ...)
-        reconstructGRLfromGR(gr, x)
-    }
+    function(x, y, ...) .fast_binary_mendoapply("intersect", x, y, ...)
 )
 
-setMethod("setdiff", c("GRanges", "GRanges"),
+### Always return a GRanges *instance* whatever GenomicRanges derivatives are
+### passed to it (e.g. GNCList or GPos), so does NOT act like an endomorphism
+### in general.
+setMethod("setdiff", c("GenomicRanges", "GenomicRanges"),
     function(x, y, ignore.strand=FALSE)
     {
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
+        x <- granges(x)
+        y <- granges(y)
         if (ignore.strand)
             strand(x) <- strand(y) <- "*"
-        mcols(x) <- mcols(y) <- NULL
         seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
         ## If merge() is going to issue a warning, we don't want to get
         ## it twice.
@@ -139,19 +160,15 @@ setMethod("setdiff", c("GRanges", "GRanges"),
     }
 )
 
-### Equivalent to 'mendoapply(setdiff, x, y)'.
+.unsupported_setdiff <- function(x, y)
+    stop("setdiff() between a ", class(x), " and a ", class(y), " object ",
+         "is not supported")
+
+setMethod("setdiff", c("GenomicRanges", "Vector"), .unsupported_setdiff)
+setMethod("setdiff", c("Vector", "GenomicRanges"), .unsupported_setdiff)
+
 setMethod("setdiff", c("GRangesList", "GRangesList"),
-    function(x, y, ...)
-    {
-        if (length(x) != length(y))
-            stop("'x' and 'y' must have the same length")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-        seqlevels(y) <- seqlevels(x)
-        xgr <- deconstructGRLintoGR(x)
-        ygr <- deconstructGRLintoGR(y)
-        gr <- callGeneric(xgr, ygr, ...)
-        reconstructGRLfromGR(gr, x)
-    }
+    function(x, y, ...) .fast_binary_mendoapply("setdiff", x, y, ...)
 )
 
 
